// File: ex5-6.cpp - a friend to the Card and Deck classes

#include <iostream>
#include <cstdlib>	// needed for rand() function
#include <string>
using namespace std;

const string value_name[13] =
{"two","three","four","five","six","seven","eight","nine","ten","jack","queen","king","ace"};

const string suit_name[4] =
{"clubs","diamonds","hearts","spades"};

const int HandSize = 5;
const int DeckSize = 52;

class Deck;  // forward declare the Deck class

class Hand
{
private:
    int card[HandSize];
public:
    Hand() { }
    void dealMe(Deck&);
    void print(const Deck&) const;
};

class Card
{
private:
    int value;
    int suit;
public:
    Card(int arg = 0) : value(arg%13), suit(arg%4) { }
    int get_value() const
    {
        return value;
    }
    int get_suit() const
    {
        return suit;
    }
    void print(void) const;
    friend void Hand::print(const Deck&) const;
};

void Card::print() const
{
    cout << value_name[value] << " of " << suit_name[suit] << endl;
}


class Deck
{
    friend class Hand;
private:
    Card d[DeckSize];
    int next_Card;
public:
    Deck();
    void shuffle();
    void deal(int=HandSize);
    void print() const;
};

Deck::Deck()
{
    for (int i = 0; i < DeckSize; i++) d[i] = Card(i);
    next_Card = 0;
}

void Deck::shuffle()
{
    int i, k;
    Card temp;
    cout << "I am shuffling the Deck\n";
    for (i = 0; i < DeckSize; i++)
    {
        k = rand() % DeckSize;
        temp = d[i];
        d[i] = d[k];
        d[k] = temp;
    }
}

void Deck::print() const
{
    for (int i = 0; i < DeckSize; i++) d[i].print();
}

void Hand::dealMe(Deck& deck)
{
    for (int i = 0; i < HandSize; i++) card[i] = deck.next_Card++;
}

void Hand::print(const Deck& deck) const
{
    cout << "here is your hand:\n";
    for (int i = 0; i < HandSize; i++) deck.d[card[i]].print();
}

int main (void)
{
    Deck poker;
    poker.shuffle();
    poker.print();
    Hand Joe;
    Hand Mary;
    Joe.dealMe(poker);
    Mary.dealMe(poker);
    cout << "\nOk, Joe ";
    Joe.print(poker);
    cout << "\nOk, Mary ";
    Mary.print(poker);
}
